package com.webonlineapp.hyjavaapexdemo.components;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.List;
import java.util.Random;

import com.hyjavacharts.hyjavaimages.ImageException;
import com.hyjavacharts.hyjavaimages.ImageFormat;
import com.hyjavacharts.hyjavaimages.ImageGenerator;
import com.hyjavacharts.hyjavaimages.ImageOptions;
import com.hyjavacharts.hyjavaimages.LibraryInfo;
import com.hyjavacharts.hyjavaimages.LibraryType;
import com.hyjavacharts.hyjavaimages.PdfOptions;
import com.vaadin.flow.component.Html;
import com.vaadin.flow.component.Tag;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.html.Image;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.radiobutton.RadioButtonGroup;
import com.vaadin.flow.component.radiobutton.RadioGroupVariant;
import com.vaadin.flow.server.StreamResource;
import com.webonlineapp.hyjavaapexdemo.util.DateUtil;

/**
 * Image generation is dependent on the HyJavaImages library.
 */

@Tag("div")
public class ApexchartsImageComponent extends VerticalLayout {
	private static final long serialVersionUID = 1L;
	
	private ImageSizes imageSizes = new ImageSizes();
	private String format = "png";
	private String imageSize = imageSizes.getDefaultImageSizeDesc();
	private int scale = 1;
	private Html imageDescription = new Html("<p></p>");
	private List<String> chartHeights = new ArrayList<String>();
	
	
	/**
	 * 
	 * @param chartOptionsJs
	 */
	public void displayChartImage(String... chartOptionsJs) {
		removeAll();
		
		add(getDemoText());
		
		RadioButtonGroup<String> imageFormatGroup = new RadioButtonGroup<>();
		imageFormatGroup.setLabel("Format");
		imageFormatGroup.setItems("png", "jpeg", "webp", "pdf");
		imageFormatGroup.addThemeVariants(RadioGroupVariant.LUMO_VERTICAL);
		imageFormatGroup.setValue(format);
		add(imageFormatGroup);
		
		RadioButtonGroup<String> sizeGroup = new RadioButtonGroup<>();
		sizeGroup.setLabel("Image Size");
		sizeGroup.setItems(imageSizes.getImageSizeDescs());
		sizeGroup.addThemeVariants(RadioGroupVariant.LUMO_VERTICAL);
		sizeGroup.setValue(imageSize);
		add(sizeGroup);
		
		RadioButtonGroup<String> scaleGroup = new RadioButtonGroup<>();
		scaleGroup.setLabel("Scale Factor");
		scaleGroup.setItems("1", "2", "3", "4");
		scaleGroup.addThemeVariants(RadioGroupVariant.LUMO_VERTICAL);
		scaleGroup.setValue(scale + "");
		scaleGroup.setTooltipText("Scales the selected image size by the scale factor. An 800 x 600 size with a scale factor of 2 will scale the image to 1600 x 1200. This is 4 times the resolution.");
		add(scaleGroup);
		
		if (imageFormatGroup.getValue().equals("pdf")) {
			scaleGroup.setEnabled(false);
		}
		else {
			scaleGroup.setEnabled(true);
		}
		
		imageFormatGroup.addValueChangeListener(ComponentValueChangeEvent -> {
			if (imageFormatGroup.getValue().equals("pdf")) {
				scaleGroup.setEnabled(false);
			}
			else {
				scaleGroup.setEnabled(true);
			}
			
			imageDescription.setHtmlContent(buildImageDescription(imageFormatGroup.getValue(), sizeGroup.getValue(), scaleGroup.getValue()));
		});
		
		sizeGroup.addValueChangeListener(ComponentValueChangeEvent -> {
			imageDescription.setHtmlContent(buildImageDescription(imageFormatGroup.getValue(), sizeGroup.getValue(), scaleGroup.getValue()));
		});
		
		scaleGroup.addValueChangeListener(ComponentValueChangeEvent -> {
			imageDescription.setHtmlContent(buildImageDescription(imageFormatGroup.getValue(), sizeGroup.getValue(), scaleGroup.getValue()));
		});
		
		Button generateButton = new Button("Generate Chart Image");
		generateButton.addThemeVariants(ButtonVariant.LUMO_PRIMARY);
		add(generateButton);
		
		imageDescription = new Html(buildImageDescription(imageFormatGroup.getValue(), sizeGroup.getValue(), scaleGroup.getValue()));
		add(imageDescription);

		generateButton.addClickListener(clickEvent -> {
			Random rand = new Random();
			removeAll();

			boolean pdf = false;
			
			ImageFormat imageFormat = null;
			format = imageFormatGroup.getValue();
			
			if (format.equals("png")) {
				imageFormat = ImageFormat.PNG;
			}
			else if (format.equals("jpeg")) {
				imageFormat = ImageFormat.JPEG;
			}
			else if (format.equals("webp")) {
				imageFormat = ImageFormat.WEBP;
			}
			else {
				pdf = true;
			}
			
			scale = Integer.valueOf(scaleGroup.getValue());
			
			imageSize = sizeGroup.getValue();
			int width = imageSizes.getWidth(imageSize);
			int height = imageSizes.getHeight(imageSize);

	        EmbeddedPdf chartPdf = null;
			Image chartImage = null;
			
			int mainChartHeight = height * Integer.parseInt(chartHeights.get(0).replace("%", "")) / 100;
			
    		ImageGenerator img = new ImageGenerator();
    		img.setChartDivSize(width + "px", mainChartHeight + "px");
    		img.setEmulateDevice(width, height);
    		img.setJavaScript(LibraryType.APEXCHARTS, "4.1.0");
    		img.setJavaScriptAdditional("https://cdn.jsdelivr.net/npm/moment@2.28.0",
    									"https://cdn.jsdelivr.net/npm/dayjs@1.11.0/dayjs.min.js", 
    									"https://cdn.jsdelivr.net/npm/dayjs@1.11.0/plugin/quarterOfYear.js");
    		img.setJavaScriptStatements("dayjs.extend(window.dayjs_plugin_quarterOfYear)");
    		
    		// the first element is always the main chart and the first to be displayed.
    		// if more than 1 set of charts, create an array of 2nd element and onwards.
    		// then use these in the setAdditionalChartOptions() method
    		if (chartOptionsJs.length > 1) {
	    		String[] chartOptionsJsArray = new String[chartOptionsJs.length - 1];
	    		String[] chartOptionsAttrArray = new String[chartOptionsJs.length - 1];
	    		for (int i = 1; i < chartOptionsJs.length; i++) {
	    			chartOptionsJsArray[i-1] = chartOptionsJs[i];
	    			int additionalHeight = height * Integer.parseInt(chartHeights.get(i).replace("%", "")) / 100;
	    			chartOptionsAttrArray[i-1] = "style=\"width: " + width + "px; height: " + additionalHeight + "px;\"";
	    		}
	    		
	    		img.setAdditionalChartOptions(chartOptionsJsArray);
	    		img.setAdditionalChartDivAttributes(chartOptionsAttrArray);
    		}
    		
    		LocalDateTime started = null;
			LocalDateTime ended = null;
    		
    		try {
    			String base64;
    			
    			if (pdf) {
    				started = LocalDateTime.now();
    				base64 = img.toBase64PDF(chartOptionsJs[0], new PdfOptions().setLandscape(true).setScale(1));
    				ended = LocalDateTime.now();
    				byte [] imageBytes = Base64.getDecoder().decode(base64);
    				StreamResource resource = new StreamResource("image" + rand.nextInt(999999999) + ".pdf", 
    	    				new ChartImageStream(imageBytes));
    				
    				chartPdf = new EmbeddedPdf(resource);
    			}
    			else {
    				started = LocalDateTime.now();
    				base64 = img.toBase64Image(chartOptionsJs[0], 
    						new ImageOptions().setFormat(imageFormat).setScale(scale));
    				ended = LocalDateTime.now();
					byte [] imageBytes = Base64.getDecoder().decode(base64); 
					StreamResource resource = new StreamResource("image" + rand.nextInt(999999999) + 
							(imageFormat == ImageFormat.PNG ? ".png" : imageFormat == ImageFormat.JPEG ? ".jpeg" : ".webp"), 
		    				new ChartImageStream(imageBytes));
					
					chartImage = new Image(resource, "");
    			}
    			
 			} catch (ImageException e) {
				e.printStackTrace();
			}
     		
    		if (pdf) {
    			add(chartPdf);
    		}
    		else {
     			chartImage.setWidth(width + "px");
    			chartImage.setHeight(height + "px");
        		add(chartImage);
    		}
       		
    		add(new Html("<p style=\"font-size: small;\"><i>Generating the base64 encoded image using the ImageGenerator class " + (pdf ? "toBase64PDF" : "toBase64Image") + "() method took " + DateUtil.formatDuration(started, ended) + "ms</i></p>"));
    		if (!pdf) {
    			add(new Html("<p style=\"font-size: small;\"><i>The ApexCharts library will render the chart based on a " + imageSize + 
    					   " window. The image size generated is " + (scale > 1 ? "then scaled to a " : "a ") + 
    					   (width * scale) + " x " + (height * scale) + " pixel " + format + " image." + "</i></p>"));

    		}
     	});
	}
	
	/**
	 * 
	 * @return
	 */
	private Html getDemoText() {
		String text = 
			"<div><h4>HyJavaImages v" + LibraryInfo.getHyJavaImagesAPIVersion() + " Library Demonstration</h4>" + 
			"<p>Select an example chart from the menu on the left and press the Generate Chart Image button.</p>" + 
			"<p>In this demo the <b>HyJavaImages</b> library generates the chart image from ApexCharts chart options displayed in the Chart Options tab above." + 
			"<br>All processing occurs on the server. No Javascript is executed on the client browser.</p>" + 
			"<p>The elapsed time to generate the base64 encoded image string is displayed below the chart image.</p></div>";
		
		return new Html(text);
	}

	/**
	 * 
	 * @param chartHeight
	 */
	public void setChartHeights(List<String> chartHeights) {
		this.chartHeights = chartHeights;
	}
	
	/**
	 * 
	 * @param format
	 * @param imageSize
	 * @param scale
	 * @return
	 */
	private String buildImageDescription(String format, String imageSize, String scale) {
		if (format.equals("pdf")) {
			return "<p style=\"font-size: small;\"><i>The ApexCharts library will render the chart based on a " + imageSize + 
				   " window. For this demonstration the pdf generated is in landscape orientation with a scale factor of 1.</i></p>";
		}
		else {
			int scaleValue = Integer.valueOf(scale);
			
			int width = imageSizes.getWidth(imageSize);
			int height = imageSizes.getHeight(imageSize);

			return "<p style=\"font-size: small;\"><i>The ApexCharts library will render the chart based on a " + imageSize + 
				   " window. The image size generated is " + (scaleValue > 1 ? "then scaled to a " : "a ") + 
				   (width * scaleValue) + " x " + (height * scaleValue) + " pixel " + format + " image." + "</i></p>";
		}
	}
	
	
	
	
	
	/**
	 * 
	 */
	private class ImageSize {
		private int width;
		private int height;
		private String desc;
		
		public ImageSize(int width, int height, String desc) {
			this.width = width;
			this.height = height;
			this.desc = desc;
		}
		public int getWidth() {
			return width;
		}
		public int getHeight() {
			return height;
		}
		public String getDesc() {
			return desc;
		}
	}
	
	/**
	 * 
	 */
	private class ImageSizes {
		List<ImageSize> imageSizeList = Arrays.asList(
				new ImageSize(800, 600, "800 x 600"),
				new ImageSize(600, 450, "600 x 450"),
				new ImageSize(1200, 900, "1200 x 900"),
				new ImageSize(1600, 1200, "1600 x 1200")
				);
		
		/**
		 * 
		 * @return
		 */
		public String[] getImageSizeDescs() {
			String[] desc = new String[imageSizeList.size()];
			
			int n = 0;
			for (ImageSize item : imageSizeList) {
				desc[n] = item.getDesc();
				n++;
			}
			
			return desc;
		}
		
		/**
		 * 
		 * @return
		 */
		public String getDefaultImageSizeDesc() {
			return imageSizeList.get(0).getDesc();
		}
		
		/**
		 * 
		 * @param desc
		 * @return
		 */
		public int getWidth(String desc) {
			for (ImageSize item : imageSizeList) {
				if (item.getDesc().equals(desc)) {
					return item.getWidth();
				}
			}
			
			return 0;
		}
		
		/**
		 * 
		 * @param desc
		 * @return
		 */
		public int getHeight(String desc) {
			for (ImageSize item : imageSizeList) {
				if (item.getDesc().equals(desc)) {
					return item.getHeight();
				}
			}
			
			return 0;
		}
	}
	
}
