package com.webonlineapp.hyjavachartsdemo12.components;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Base64;
import java.util.List;
import java.util.Random;

import com.hyjavacharts.hyjavaimages.ImageException;
import com.hyjavacharts.hyjavaimages.ImageFormat;
import com.hyjavacharts.hyjavaimages.ImageGenerator;
import com.hyjavacharts.hyjavaimages.ImageOptions;
import com.hyjavacharts.hyjavaimages.LibraryInfo;
import com.hyjavacharts.hyjavaimages.LibraryType;
import com.hyjavacharts.hyjavaimages.PdfOptions;
import com.vaadin.flow.component.Html;
import com.vaadin.flow.component.Tag;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.html.Image;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.radiobutton.RadioButtonGroup;
import com.vaadin.flow.component.radiobutton.RadioGroupVariant;
import com.vaadin.flow.server.StreamResource;
import com.webonlineapp.hyjavachartsdemo12.Properties;
import com.webonlineapp.hyjavachartsdemo12.util.DateUtil;

/**
 * Image generation is dependent on the HyJavaImages library.
 */

@Tag("div")
public class HighchartImageComponent extends VerticalLayout {
	private static final long serialVersionUID = 1L;
	
	private ImageSizes imageSizes = new ImageSizes();
	private String format = "png";
	private String imageSize = imageSizes.getDefaultImageSizeDesc();
	private int scale = 1;
	private Html imageDescription = new Html("<p></p>");
	
	
	/**
	 * 
	 * @param globalOptionsJs
	 * @param chartOptionsJs
	 */
	public void displayChartImage(String globalOptionsJs, String chartOptionsJs, String demoJavaScriptPre, String demoJavaScriptPost) {
		removeAll();
		
		add(getDemoText());
		
		RadioButtonGroup<String> imageFormatGroup = new RadioButtonGroup<>();
		imageFormatGroup.setLabel("Format");
		imageFormatGroup.setItems("png", "jpeg", "webp", "pdf");
		imageFormatGroup.addThemeVariants(RadioGroupVariant.LUMO_VERTICAL);
		imageFormatGroup.setValue(format);
		add(imageFormatGroup);
		
		RadioButtonGroup<String> sizeGroup = new RadioButtonGroup<>();
		sizeGroup.setLabel("Image Size");
		sizeGroup.setItems(imageSizes.getImageSizeDescs());
		sizeGroup.addThemeVariants(RadioGroupVariant.LUMO_VERTICAL);
		sizeGroup.setValue(imageSize);
		add(sizeGroup);
		
		RadioButtonGroup<String> scaleGroup = new RadioButtonGroup<>();
		scaleGroup.setLabel("Scale Factor");
		scaleGroup.setItems("1", "2", "3", "4");
		scaleGroup.addThemeVariants(RadioGroupVariant.LUMO_VERTICAL);
		scaleGroup.setValue(scale + "");
		scaleGroup.setTooltipText("Scales the selected image size by the scale factor. An 800 x 600 size with a scale factor of 2 will scale the image to 1600 x 1200. This is 4 times the resolution.");
		add(scaleGroup);
		
		if (imageFormatGroup.getValue().equals("pdf")) {
			scaleGroup.setEnabled(false);
		}
		else {
			scaleGroup.setEnabled(true);
		}
		
		imageFormatGroup.addValueChangeListener(ComponentValueChangeEvent -> {
			if (imageFormatGroup.getValue().equals("pdf")) {
				scaleGroup.setEnabled(false);
			}
			else {
				scaleGroup.setEnabled(true);
			}
			
			imageDescription.setHtmlContent(buildImageDescription(imageFormatGroup.getValue(), sizeGroup.getValue(), scaleGroup.getValue()));
		});
		
		sizeGroup.addValueChangeListener(ComponentValueChangeEvent -> {
			imageDescription.setHtmlContent(buildImageDescription(imageFormatGroup.getValue(), sizeGroup.getValue(), scaleGroup.getValue()));
		});
		
		scaleGroup.addValueChangeListener(ComponentValueChangeEvent -> {
			imageDescription.setHtmlContent(buildImageDescription(imageFormatGroup.getValue(), sizeGroup.getValue(), scaleGroup.getValue()));
		});
		
		Button generateButton = new Button("Generate Chart Image");
		generateButton.addThemeVariants(ButtonVariant.LUMO_PRIMARY);
		add(generateButton);
		
		imageDescription = new Html(buildImageDescription(imageFormatGroup.getValue(), sizeGroup.getValue(), scaleGroup.getValue()));
		add(imageDescription);

		generateButton.addClickListener(clickEvent -> {
			Random rand = new Random();
			removeAll();

			boolean pdf = false;
			
			ImageFormat imageFormat = null;
			format = imageFormatGroup.getValue();
			
			if (format.equals("png")) {
				imageFormat = ImageFormat.PNG;
			}
			else if (format.equals("jpeg")) {
				imageFormat = ImageFormat.JPEG;
			}
			else if (format.equals("webp")) {
				imageFormat = ImageFormat.WEBP;
			}
			else {
				pdf = true;
			}
			
			scale = Integer.valueOf(scaleGroup.getValue());
			
			imageSize = sizeGroup.getValue();
			int width = imageSizes.getWidth(imageSize);
			int height = imageSizes.getHeight(imageSize);

	        EmbeddedPdf chartPdf = null;
			Image chartImage = null;
			
    		ImageGenerator img = new ImageGenerator();
    		img.setChartDivSize(width + "px", height + "px");
    		img.setEmulateDevice(width, height);
    		img.setJavaScriptAlternate(LibraryType.HIGHCHARTS, 
    				Properties.HIGHCHARTS_VERSION + "highcharts.js", 
    				Properties.HIGHCHARTS_VERSION + "highcharts-more.js", 
    				Properties.HIGHCHARTS_VERSION + "highcharts-3d.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/networkgraph.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/drilldown.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/funnel.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/heatmap.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/exporting.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/solid-gauge.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/sankey.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/wordcloud.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/histogram-bellcurve.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/treemap.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/bullet.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/tilemap.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/pareto.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/boost.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/datagrouping.js",
    				Properties.HIGHCHARTS_VERSION + "modules/windbarb.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/xrange.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/variwide.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/annotations.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/variable-pie.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/vector.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/sunburst.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/data.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/pattern-fill.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/parallel-coordinates.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/venn.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/cylinder.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/funnel3d.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/pyramid3d.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/dependency-wheel.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/item-series.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/timeline.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/organization.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/dumbbell.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/lollipop.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/arc-diagram.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/series-label.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/sonification.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/pictorial.js", 
    				Properties.HIGHCHARTS_VERSION + "modules/treegraph.js", 
    				"https://www.hyjavacharts.com/content/js/highcharts-regression.js");
    		if (!demoJavaScriptPre.equals("")) {
    			img.setJavaScriptStatements(demoJavaScriptPre);
    		}
    		
    		LocalDateTime started = null;
			LocalDateTime ended = null;
    		
    		try {
    			String base64;
     			
    			if (pdf) {
    				started = LocalDateTime.now();
    				base64 = img.toBase64PDF(globalOptionsJs, chartOptionsJs, 
    											new PdfOptions().setLandscape(true).setScale(1));
    				ended = LocalDateTime.now();
    				byte [] imageBytes = Base64.getDecoder().decode(base64);
    				StreamResource resource = new StreamResource("image" + rand.nextInt(999999999) + ".pdf", 
    	    				new ChartImageStream(imageBytes));
    				
    				chartPdf = new EmbeddedPdf(resource);
    			}
    			else {
    				started = LocalDateTime.now();
					base64 = img.toBase64Image(globalOptionsJs, chartOptionsJs, 
												new ImageOptions().setFormat(imageFormat).setScale(scale));
					ended = LocalDateTime.now();
					byte [] imageBytes = Base64.getDecoder().decode(base64); 
					StreamResource resource = new StreamResource("image" + rand.nextInt(999999999) + 
							(imageFormat == ImageFormat.PNG ? ".png" : imageFormat == ImageFormat.JPEG ? ".jpeg" : ".webp"), 
		    				new ChartImageStream(imageBytes));
					
					chartImage = new Image(resource, "");
    			}

			} catch (ImageException e) {
				e.printStackTrace();
			}

    		if (pdf) {
    			add(chartPdf);
    		}
    		else {
     			chartImage.setWidth(width + "px");
    			chartImage.setHeight(height + "px");
        		add(chartImage);
    		}

    		add(new Html("<p style=\"font-size: small;\"><i>Generating the base64 encoded image using the ImageGenerator class " + (pdf ? "toBase64PDF" : "toBase64Image") + "() method took " + DateUtil.formatDuration(started, ended) + "ms</i></p>"));
    		if (!pdf) {
    			add(new Html("<p style=\"font-size: small;\"><i>The Highcharts library will render the chart based on a " + imageSize + 
				   " window. The image size generated is " + (scale > 1 ? "then scaled to a " : "a ") + 
				   (width * scale) + " x " + (height * scale) + " pixel " + format + " image." + "</i></p>"));
    		}
     	});
	}
	
	/**
	 * 
	 * @return
	 */
	private Html getDemoText() {
		String text = 
			"<div><h4>HyJavaImages v" + LibraryInfo.getHyJavaImagesAPIVersion() + " Library Demonstration</h4>" + 
			"<p>Select an example chart from the menu on the left and press the Generate Chart Image button.</p>" + 
			"<p>In this demo the <b>HyJavaImages</b> library generates the chart image from Highcharts chart options displayed in the Chart Options tab above." + 
			"<br>All processing occurs on the server. No Javascript is executed on the client browser.</p>" + 
			"<p>The elapsed time to generate the base64 encoded image string is displayed below the chart image.</p></div>";
		
		return new Html(text);
	}
	
	/**
	 * 
	 * @param format
	 * @param imageSize
	 * @param scale
	 * @return
	 */
	private String buildImageDescription(String format, String imageSize, String scale) {
		if (format.equals("pdf")) {
			return "<p style=\"font-size: small;\"><i>The Highcharts library will render the chart based on a " + imageSize + 
				   " window. For this demonstration the pdf generated is in landscape orientation with a scale factor of 1.</i></p>";
		}
		else {
			int scaleValue = Integer.valueOf(scale);
			
			int width = imageSizes.getWidth(imageSize);
			int height = imageSizes.getHeight(imageSize);

			return "<p style=\"font-size: small;\"><i>The Highcharts library will render the chart based on a " + imageSize + 
				   " window. The image size generated is " + (scaleValue > 1 ? "then scaled to a " : "a ") + 
				   (width * scaleValue) + " x " + (height * scaleValue) + " pixel " + format + " image." + "</i></p>";
		}
	}
	
	
	

	/**
	 * 
	 */
	private class ImageSize {
		private int width;
		private int height;
		private String desc;
		
		public ImageSize(int width, int height, String desc) {
			this.width = width;
			this.height = height;
			this.desc = desc;
		}
		public int getWidth() {
			return width;
		}
		public int getHeight() {
			return height;
		}
		public String getDesc() {
			return desc;
		}
	}
	
	/**
	 * 
	 */
	private class ImageSizes {
		List<ImageSize> imageSizeList = Arrays.asList(
				new ImageSize(800, 600, "800 x 600"),
				new ImageSize(600, 450, "600 x 450"),
				new ImageSize(1200, 900, "1200 x 900"),
				new ImageSize(1600, 1200, "1600 x 1200")
				);
		
		/**
		 * 
		 * @return
		 */
		public String[] getImageSizeDescs() {
			String[] desc = new String[imageSizeList.size()];
			
			int n = 0;
			for (ImageSize item : imageSizeList) {
				desc[n] = item.getDesc();
				n++;
			}
			
			return desc;
		}
		
		/**
		 * 
		 * @return
		 */
		public String getDefaultImageSizeDesc() {
			return imageSizeList.get(0).getDesc();
		}
		
		/**
		 * 
		 * @param desc
		 * @return
		 */
		public int getWidth(String desc) {
			for (ImageSize item : imageSizeList) {
				if (item.getDesc().equals(desc)) {
					return item.getWidth();
				}
			}
			
			return 0;
		}
		
		/**
		 * 
		 * @param desc
		 * @return
		 */
		public int getHeight(String desc) {
			for (ImageSize item : imageSizeList) {
				if (item.getDesc().equals(desc)) {
					return item.getHeight();
				}
			}
			
			return 0;
		}
	}
	
}
