package com.webonlineapp.hyjavachartsdemo12.examplecharts;

import java.util.Arrays;

import com.hyjavacharts.chart.Highchart;
import com.hyjavacharts.model.common.Color;
import com.hyjavacharts.model.common.General;
import com.hyjavacharts.model.common.LinearGradient;
import com.hyjavacharts.model.common.Stop;
import com.hyjavacharts.model.highcharts.ChartOptions;
import com.hyjavacharts.model.highcharts.XAxis;
import com.hyjavacharts.model.highcharts.YAxis;
import com.hyjavacharts.model.highcharts.constants.AxisType;
import com.hyjavacharts.model.highcharts.constants.DashStyle;
import com.hyjavacharts.model.highcharts.series.SeriesLine;
import com.hyjavacharts.model.highcharts.series.seriesline.Zones;
import com.webonlineapp.hyjavachartsdemo12.charts.DemoChartBase;

public class Forecast extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highchart configure() {
		Highchart highChart = new Highchart();
		ChartOptions chartOptions = highChart.getChartOptions();
	    
		chartOptions.getTitle().setText("Hourly forecast temperatures in Reykjavík, Iceland");
		chartOptions.getSubtitle().setText("Dotted line typically signifies prognosis");

		XAxis xAxis = new XAxis();
		xAxis.setType(AxisType.DATETIME);
		xAxis.getPlotLinesSingle().setColor(new Color("#4840d6")).setWidth(2).setZIndex(2).setDashStyle(DashStyle.DASH)
			.getLabel().setText("Current time").setRotation(0).setY(20).getStyle().setColor(new Color("#333333"));
		xAxis.getPlotLinesSingle().set("value", new General("today"));
		chartOptions.getXAxis().add(xAxis);

		YAxis yAxi = new YAxis();
		yAxi.getTitle().setText("Temperature (°C)");
		chartOptions.getYAxis().add(yAxi);

		chartOptions.getLegend().setEnabled(false);
		chartOptions.getTooltip().setValueSuffix("°C");

		SeriesLine seriesLine = new SeriesLine();
		seriesLine.setName("Temperature in Reykjavík");
		seriesLine.setDataAsJsValue(new General("data"));
		seriesLine.getLabel().setEnabled(false);
		seriesLine.setZoneAxis("x").setLineWidth(4);
		seriesLine.getMarker().setLineWidth(2).setLineColor(new Color("#4840d6")).setFillColor(new Color("#fff"));
		seriesLine.setColor(new Color(new LinearGradient(0, 0, 0, 1), 
										Arrays.asList(new Stop(0, new Color("#fa4fed")), 
													  new Stop(1, new Color("#5897ff")))));
		Zones valueZone = new Zones();
		valueZone.set("value", new General("today"));
		seriesLine.setZones(Arrays.asList(valueZone, new Zones().setDashStyle(DashStyle.DOT)));
		chartOptions.getSeries().add(seriesLine);

		this.setDemoJavaScriptPre("(async () => { "
				+ "    const json = await fetch( "
				+ "            'https://api.met.no/weatherapi/locationforecast/2.0/compact?lat=64.128288&lon=-21.827774' "
				+ "        ).then(response => response.json()), "
				+ "        data = json.properties.timeseries.slice(0, 10).map(el => "
				+ "            [ "
				+ "                new Date(el.time).getTime(), "
				+ "                el.data.instant.details.air_temperature "
				+ "            ]), "
				+ "        todayDate = new Date(), "
				+ "        today = todayDate.getTime() - todayDate.getTimezoneOffset() * 60 * 1000;");
		
		this.setDemoJavaScriptPost("})();");
		return highChart;
	}
	
}
