package com.webonlineapp.hyjavachartsdemo12.examplecharts;

import java.util.Arrays;

import com.hyjavacharts.chart.Highchart;
import com.hyjavacharts.model.common.Color;
import com.hyjavacharts.model.common.LinearGradient;
import com.hyjavacharts.model.common.Stop;
import com.hyjavacharts.model.highcharts.ChartOptions;
import com.hyjavacharts.model.highcharts.XAxis;
import com.hyjavacharts.model.highcharts.YAxis;
import com.hyjavacharts.model.highcharts.constants.AxisType;
import com.hyjavacharts.model.highcharts.series.SeriesLine;
import com.hyjavacharts.model.highcharts.series.seriesline.Data;
import com.webonlineapp.hyjavachartsdemo12.charts.DemoChartBase;

public class LogarithmicAxis extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highchart configure() {
		Highchart highChart = new Highchart();
		ChartOptions chartOptions = highChart.getChartOptions();
	    
		chartOptions.getTitle().setText("Growth of Internet Users Worldwide (logarithmic scale)");
		chartOptions.getAccessibility().getPoint().setValueDescriptionFormat("{xDescription}{separator}{value} million(s)");
		
		XAxis xAxis = new XAxis().setCategories(Arrays.asList("1995", "2000", "2005", "2010", "2015", "2020", "2023"));
		xAxis.getTitle().setText("Year");
		chartOptions.getXAxis().add(xAxis);
		
		YAxis yAxis = new YAxis().setType(AxisType.LOGARITHMIC);
		yAxis.getTitle().setText("Number of Internet Users (in millions)");
		chartOptions.getYAxis().add(yAxis);

		chartOptions.getTooltip()
			.setHeaderFormat("<b>{series.name}</b><br />")
			.setPointFormat("{point.y} million(s)");

		SeriesLine seriesLine = new SeriesLine();
		seriesLine.setName("Internet Users");
		seriesLine.getLabel().setEnabled(false);
		
//		seriesLine.setKeys(Arrays.asList("y", "color"));
//		seriesLine.setDataAsJsValue(new General("[[16, '#0000ff'], [361, '#8d0073'], [1018, '#ba0046'], [2025, '#d60028'], [3192, '#eb0014'], [4673, '#fb0004'], [5200, '#ff0000']]"));
		seriesLine.setDataAsArrayObject(Arrays.asList(
				new Data().setY(16).setColor(new Color("#0000ff")),
				new Data().setY(361).setColor(new Color("#8d0073")),
				new Data().setY(1018).setColor(new Color("#ba0046")),
				new Data().setY(2025).setColor(new Color("#d60028")),
				new Data().setY(3192).setColor(new Color("#eb0014")),
				new Data().setY(4673).setColor(new Color("#fb0004")),
				new Data().setY(5200).setColor(new Color("#ff0000"))
				));

		seriesLine.setColor(new Color(new LinearGradient(0, 0, 1, 0), 
									  Arrays.asList(new Stop(0, new Color("#0000ff")), 
											  		new Stop(1, new Color("#ff0000")))));
		chartOptions.getSeries().add(seriesLine);
		
		return highChart;
	}

}
