package com.webonlineapp.hyjavachartsdemo12.examplecharts;

import java.util.Arrays;

import com.hyjavacharts.chart.Highchart;
import com.hyjavacharts.model.highcharts.ChartOptions;
import com.hyjavacharts.model.highcharts.YAxis;
import com.hyjavacharts.model.highcharts.constants.ChartType;
import com.hyjavacharts.model.highcharts.constants.Stacking;
import com.hyjavacharts.model.highcharts.series.SeriesArea;
import com.webonlineapp.hyjavachartsdemo12.charts.DemoChartBase;

public class PercentageArea extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highchart configure() {
		Highchart highChart = new Highchart();
		ChartOptions chartOptions = highChart.getChartOptions();
		
		chartOptions.getChart().setType(ChartType.AREA);
		chartOptions.getTitle().setUseHTML(true).setText("Countries/regions with highest Gt CO<sub>2</sub>-emissions");
		chartOptions.getSubtitle().setText("Source: <a href=\"https://energiogklima.no/klimavakten/land-med-hoyest-utslipp/\" target=\"_blank\">Energi og Klima</a>");

		chartOptions.getAccessibility().getPoint()
			.setValueDescriptionFormat("{index}. {point.category}, {point.y:,.1f} billions, {point.percentage:.1f}%.");

		YAxis yAxis = new YAxis();
		yAxis.getLabels().setFormat("{value}%");
		yAxis.getTitle().setText(null);
		chartOptions.getYAxis().add(yAxis);

		chartOptions.getTooltip()
			.setPointFormat("<span style=\"color:{series.color}\">{series.name}</span>: <b>{point.percentage:.1f}%</b> ({point.y:,.1f} billion Gt)<br/>")
			.setSplit(true);

		chartOptions.getPlotOptions().getArea()
			.setPointStart(1990)
			.getLabel().getStyle().setFontSize("1.4em").setOpacity(0.4);
		chartOptions.getPlotOptions().getArea()
			.setStacking(Stacking.PERCENT)
			.getMarker().setEnabled(false);
		
		SeriesArea seriesArea = new SeriesArea();
		seriesArea.setName("China");
		seriesArea.setDataAsArrayNumber(Arrays.asList(2.5, 2.6, 2.7, 2.9, 3.1, 3.4, 3.5, 3.5, 3.4, 3.6, 3.6, 3.7,
											            4.1, 4.8, 5.2, 5.9, 6.5, 7, 7.5, 7.9, 8.6, 9.5, 9.8, 10, 10,
											            9.9, 9.8, 10, 10.4, 10.7, 10.9, 11.3, 11.4, 11.9));
		chartOptions.getSeries().add(seriesArea);
		
		seriesArea = new SeriesArea();
		seriesArea.setName("USA");
		seriesArea.setDataAsArrayNumber(Arrays.asList(5.1, 5.1, 5.2, 5.3, 5.4, 5.4, 5.6, 5.7, 5.7, 5.8, 6, 5.9, 5.9,
											            6, 6.1, 6.1, 6.1, 6.1, 5.9, 5.5, 5.7, 5.5, 5.3, 5.5, 5.5, 5.4,
											            5.3, 5.2, 5.4, 5.3, 4.7, 5, 5.1, 4.9));
		chartOptions.getSeries().add(seriesArea);
		
		seriesArea = new SeriesArea();
		seriesArea.setName("EU");
		seriesArea.setDataAsArrayNumber(Arrays.asList(3.9, 3.8, 3.7, 3.6, 3.6, 3.6, 3.7, 3.7, 3.6, 3.6, 3.6, 3.7, 3.7,
											            3.7, 3.8, 3.7, 3.7, 3.7, 3.6, 3.3, 3.4, 3.3, 3.3, 3.2, 3, 3.1,
											            3.1, 3.1, 3, 2.9, 2.6, 2.8, 2.8, 2.6));
		chartOptions.getSeries().add(seriesArea);
		
		seriesArea = new SeriesArea();
		seriesArea.setName("India");
		seriesArea.setDataAsArrayNumber(Arrays.asList(0.6, 0.6, 0.7, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1, 1, 1, 1, 1.1,
											            1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 2, 2.1, 2.2, 2.4,
											            2.4, 2.6, 2.6, 2.4, 2.7, 2.8, 3.1));
		chartOptions.getSeries().add(seriesArea);

		return highChart;
	}
	
}
