package com.webonlineapp.hyjavachartsdemo12.examplecharts;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.hyjavacharts.chart.Highchart;
import com.hyjavacharts.model.common.Color;
import com.hyjavacharts.model.highcharts.ChartOptions;
import com.hyjavacharts.model.highcharts.XAxis;
import com.hyjavacharts.model.highcharts.YAxis;
import com.hyjavacharts.model.highcharts.constants.ChartType;
import com.hyjavacharts.model.highcharts.constants.Symbol;
import com.hyjavacharts.model.highcharts.series.SeriesSpline;
import com.hyjavacharts.model.highcharts.series.seriesspline.Data;
import com.webonlineapp.hyjavachartsdemo12.charts.DemoChartBase;

public class SplineWithSymbols extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highchart configure() {
		Highchart highChart = new Highchart();
		ChartOptions chartOptions = highChart.getChartOptions();

		chartOptions.getChart().setType(ChartType.SPLINE);
		chartOptions.getTitle().setText("Monthly Average Temperature");
		chartOptions.getSubtitle().setText("Source: <a href=\"https://en.wikipedia.org/wiki/List_of_cities_by_average_temperature\" target=\"_blank\">Wikipedia.com</a>");

		XAxis xAxis = new XAxis();
		xAxis.setCrosshair(true).setCategories(
			Arrays.asList("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"));
		xAxis.getAccessibility().setDescription("Months of the year");
		chartOptions.getXAxis().add(xAxis);

		YAxis yAxis = new YAxis();
		yAxis.getTitle().setText("Temperature");
		yAxis.getLabels().setFormat("{value}°");
		chartOptions.getYAxis().add(yAxis);

		chartOptions.getTooltip().setShared(true);
	    
		chartOptions.getPlotOptions().getSpline().getMarker().setRadius(4).setLineColor(new Color("#666666")).setLineWidth(1);

		SeriesSpline seriesSpline = new SeriesSpline();
		seriesSpline.setName("Tokyo").getMarker().setSymbol(Symbol.SQUARE);
		seriesSpline.setDataAsArrayObject(getSeriesData("Tokyo"));
		chartOptions.getSeries().add(seriesSpline);
		
		seriesSpline = new SeriesSpline();
		seriesSpline.setName("Bergen").getMarker().setSymbol(Symbol.DIAMOND);
		seriesSpline.setDataAsArrayObject(getSeriesData("Bergen"));
		chartOptions.getSeries().add(seriesSpline);
		
		return highChart;
	}
	
	/**
	 * 
	 * @param series
	 * @return
	 */
	private List<Data> getSeriesData(String series) {
		List<Data> array = new ArrayList<Data>();
		
		if (series.equals("Tokyo")) {
			Data d = new Data();
			d.setY(26.4);
			d.getMarker().setSymbol("url(https://www.hyjavacharts.com/content/images/sun.png)");
			d.getAccessibility().setDescription("Sunny symbol, this is the warmest point in the chart.");
			
			array.add(new Data().setY(5.2));
			array.add(new Data().setY(5.7));
			array.add(new Data().setY(8.7));
			array.add(new Data().setY(13.9));
			array.add(new Data().setY(18.2));
			array.add(new Data().setY(21.4));
			array.add(new Data().setY(25.0));
			array.add(d);
			array.add(new Data().setY(22.8));
			array.add(new Data().setY(17.5));
			array.add(new Data().setY(12.1));
			array.add(new Data().setY(7.6));
		}
		else {
			Data d = new Data();
			d.setY(1.5);
			d.getMarker().setSymbol("url(https://www.hyjavacharts.com/content/images/snow.png)");
			d.getAccessibility().setDescription("Snowy symbol, this is the coldest point in the chart.");
			
			array.add(d);
			array.add(new Data().setY(1.6));
			array.add(new Data().setY(3.3));
			array.add(new Data().setY(5.9));
			array.add(new Data().setY(10.5));
			array.add(new Data().setY(13.5));
			array.add(new Data().setY(14.5));
			array.add(new Data().setY(14.4));
			array.add(new Data().setY(11.5));
			array.add(new Data().setY(8.7));
			array.add(new Data().setY(4.7));
			array.add(new Data().setY(2.6));
		}
		
    	return array;
    }
}
