package com.webonlineapp.hyjavachartsdemo12.examplecharts;

import com.hyjavacharts.chart.Highchart;
import com.hyjavacharts.model.common.Function;
import com.hyjavacharts.model.common.SVGPathArray;
import com.hyjavacharts.model.common.SVGPathCommand;
import com.hyjavacharts.model.highcharts.ChartOptions;
import com.hyjavacharts.model.highcharts.YAxis;
import com.hyjavacharts.model.highcharts.constants.ChartType;
import com.hyjavacharts.model.highcharts.series.SeriesGauge;
import com.webonlineapp.hyjavachartsdemo12.charts.DemoChartBase;
import com.webonlineapp.hyjavachartsdemo12.services.RestService;

public class Gauge extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highchart configure() {
		Highchart highChart = new Highchart();
		ChartOptions chartOptions = highChart.getChartOptions();
	    
		chartOptions.getChart().setType(ChartType.GAUGE).getEvents().setLoad(getLoadFunction());
		
		chartOptions.getTitle().setText("Gauge");
		chartOptions.getSubtitle().setText("(will stop after 60 seconds)");
		
		chartOptions.getPaneSingle()
			.setStartAngle(-120)
			.setEndAngle(120);
		
		YAxis yAxis = new YAxis();
		yAxis.setMin(0).setMax(100);
		chartOptions.getYAxis().add(yAxis);
		
		SeriesGauge sg = new SeriesGauge();
		sg.getPivot().setRadius(0);
		sg.getDial().setPath(new SVGPathArray(new SVGPathCommand("M", 0, -10), 
											  new SVGPathCommand("l", 100, 0),
											  new SVGPathCommand("l", 0, -10),
											  new SVGPathCommand("l", 20, 20),
											  new SVGPathCommand("l", -20, 20),
											  new SVGPathCommand("l", 0, -10),
											  new SVGPathCommand("l", -100, 0)
											  ));
		chartOptions.getSeries().add(sg);
		
		// Some of our example charts use a REST service to retrieve data.
		chartOptions.getData()
			.setDataRefreshRate(3)
			.setComplete(new Function("runcounter++; if (runcounter > 20) { chart.destroy(); }"));

		return highChart;
	}
	
	/**
	 * 
	 * @return
	 */
	private Function getLoadFunction() {
		return new Function(
			"runcounter = 0; " + 
			"this.update({ data: { enablePolling: true, csvURL: '" + RestService.getURI() + "/GaugeCSVData" + "'} });");
	}

}
