package com.webonlineapp.hyjavachartsdemo12.examplecharts;

import java.util.Arrays;

import com.hyjavacharts.chart.Highchart;
import com.hyjavacharts.model.common.Color;
import com.hyjavacharts.model.highcharts.ChartOptions;
import com.hyjavacharts.model.highcharts.XAxis;
import com.hyjavacharts.model.highcharts.YAxis;
import com.hyjavacharts.model.highcharts.constants.AxisType;
import com.hyjavacharts.model.highcharts.constants.ChartType;
import com.hyjavacharts.model.highcharts.constants.HorizontalAlign;
import com.hyjavacharts.model.highcharts.constants.LabelsOverflow;
import com.hyjavacharts.model.highcharts.series.SeriesSpline;
import com.hyjavacharts.model.highcharts.yaxis.PlotBands;
import com.webonlineapp.hyjavachartsdemo12.charts.DemoChartBase;

public class SplineWithPlotBands extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highchart configure() {
		Highchart highChart = new Highchart();
		ChartOptions chartOptions = highChart.getChartOptions();
		
		chartOptions.getChart().setType(ChartType.SPLINE)
			.getScrollablePlotArea().setMinWidth(600).setScrollPositionX(1);
		
		chartOptions.getTitle().setText("Wind speed during a day").setAlign(HorizontalAlign.LEFT);
		chartOptions.getSubtitle().setText("29th of February, 2024 at two locations in Vik i Sogn, Norway").setAlign(HorizontalAlign.LEFT);

		XAxis xAxis = new XAxis();
		xAxis.setType(AxisType.DATETIME).getLabels().setOverflow(LabelsOverflow.JUSTIFY);
		chartOptions.getXAxis().add(xAxis);
		
		YAxis yAxis = new YAxis();
		yAxis.getTitle().setText("Wind speed (m/s)");
		yAxis.setMinorGridLineWidth(0).setGridLineWidth(0).setAlternateGridColor(null);

		PlotBands pb = new PlotBands();
		pb.setFrom(0.3).setTo(1.5).setColor(new Color(68, 170, 213, 0.1)).getLabel().setText("Light air").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);

		pb = new PlotBands();
		pb.setFrom(1.5).setTo(3.3).setColor(new Color(0, 0, 0, 0)).getLabel().setText("Light breeze").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);
		
		pb = new PlotBands();
		pb.setFrom(3.3).setTo(5.5).setColor(new Color(68, 170, 213, 0.1)).getLabel().setText("Gentle breeze").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);

		pb = new PlotBands();
		pb.setFrom(5.5).setTo(8).setColor(new Color(0, 0, 0, 0)).getLabel().setText("Moderate breeze").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);

		
		pb = new PlotBands();
		pb.setFrom(8).setTo(11).setColor(new Color(68, 170, 213, 0.1)).getLabel().setText("Fresh breeze").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);
		
		pb = new PlotBands();
		pb.setFrom(11).setTo(14).setColor(new Color(0, 0, 0, 0)).getLabel().setText("Strong breeze").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);

		pb = new PlotBands();
		pb.setFrom(11).setTo(14).setColor(new Color(68, 170, 213, 0.1)).getLabel().setText("Near gale").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);

		pb = new PlotBands();
		pb.setFrom(11).setTo(14).setColor(new Color(0, 0, 0, 0)).getLabel().setText("Fresh gale").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);
		
		pb = new PlotBands();
		pb.setFrom(14).setTo(15).setColor(new Color().setRGBA(68, 170, 213, 0.1)).getLabel().setText("Strong gale").getStyle().setColor(new Color("#606060"));
		yAxis.getPlotBands().add(pb);
		
		chartOptions.getYAxis().add(yAxis);
		
		chartOptions.getTooltip().setValueSuffix(" m/s");

		chartOptions.getPlotOptions().getSpline().setLineWidth(4).getStates().getHover().setLineWidth(5);
		chartOptions.getPlotOptions().getSpline().setPointInterval(3600000)
			.setPointStart("2014-02-29")
			.getMarker().setEnabled(false);

		chartOptions.getNavigation().getMenuItemStyle().setFontSize("10px");	
		
		SeriesSpline seriesSpline = new SeriesSpline();
		seriesSpline.setName("Hestavollane");
		seriesSpline.setDataAsArrayNumber(Arrays.asList(
				12.9, 13.8, 10.2, 8.4, 10.0, 9.2, 10.0,
	            12.2, 13.2, 12.7, 12.5, 11.4, 10.4,
	            7.9, 8.0, 11.4, 11.5, 12.0, 12.0,
	            10.4, 11.2, 11.5, 12.2, 11.5, 8.3));
		chartOptions.getSeries().add(seriesSpline);
		
		seriesSpline = new SeriesSpline();
		seriesSpline.setName("Vik");
		seriesSpline.setDataAsArrayNumber(Arrays.asList(
				null, 1.3, 1.1, 0.8, 1.8, 1.7, 0.8,
	            0.8, 1.0, 1.0, 1.0, 0.8, 1.4,
	            1.3, 2.9, 6.1, 6.4, 6.6, 6.4,
	            6.3, 5.4, 3.9, 3.0, 1.7, 1.4));
		chartOptions.getSeries().add(seriesSpline);
		
		return highChart;
	}
	
}
