package com.webonlineapp.hyjavachartsdemo12.examplecharts;

import java.util.Arrays;

import com.hyjavacharts.chart.Highchart;
import com.hyjavacharts.model.common.Color;
import com.hyjavacharts.model.common.General;
import com.hyjavacharts.model.common.LinearGradient;
import com.hyjavacharts.model.common.NullOption;
import com.hyjavacharts.model.common.Stop;
import com.hyjavacharts.model.highcharts.ChartOptions;
import com.hyjavacharts.model.highcharts.XAxis;
import com.hyjavacharts.model.highcharts.YAxis;
import com.hyjavacharts.model.highcharts.constants.AxisType;
import com.hyjavacharts.model.highcharts.constants.SeriesType;
import com.hyjavacharts.model.highcharts.constants.ZoomType;
import com.hyjavacharts.model.highcharts.series.SeriesArea;
import com.webonlineapp.hyjavachartsdemo12.charts.DemoChartBase;
import com.webonlineapp.hyjavachartsdemo12.services.RestService;

import jakarta.ws.rs.client.WebTarget;

public class TimeSeriesZoomable extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highchart configure() {
		Highchart highChart = new Highchart();
		ChartOptions chartOptions = highChart.getChartOptions();
	    
		chartOptions.getChart().getZooming().setType(ZoomType.X);
		chartOptions.getTitle().setText("USD to EUR exchange rate over time");
		chartOptions.getSubtitle().setText("Click and drag in the plot area to zoom in");
	
		chartOptions.getXAxis().add(new XAxis().setType(AxisType.DATETIME));
		
		YAxis yAxi = new YAxis();
		yAxi.getTitle().setText("Exchange rate");
		chartOptions.getYAxis().add(yAxi);
		
		chartOptions.getLegend().setEnabled(false);

		chartOptions.getPlotOptions().getArea().setColor(new Color(new LinearGradient(0, 0, 0, 1), 
											                Arrays.asList(new Stop(0, new Color(199, 113, 243)), 
											                              new Stop(0.7, new Color(76, 175, 254)))));
		chartOptions.getPlotOptions().getArea().getMarker().setRadius(2);
		chartOptions.getPlotOptions().getArea().setLineWidth(1);
		chartOptions.getPlotOptions().getArea().getStates().getHover().setLineWidth(1);
		chartOptions.getPlotOptions().getArea().setThreshold(NullOption.NULL_NUMBER);
		
		SeriesArea seriesArea = new SeriesArea();
		seriesArea.getLabel().setEnabled(false);
		seriesArea.setType(SeriesType.AREA).setName("USD to EUR");
		seriesArea.setDataAsJsValue(getSeriesData());
		chartOptions.getSeries().add(seriesArea);
		
		return highChart;
	}

	/**
	 * 
	 * @return
	 */
	private General getSeriesData() {
		// Some of our example charts use a REST service to retrieve data.
		WebTarget webTarget = new RestService().getWebTarget();
		
		return new General(webTarget.path("TimeSeriesZoomableData").request().get(String.class));
    }
	
}
