package com.webonlineapp.hyjavachartsdemo12.examplecharts;

import java.util.Arrays;

import com.hyjavacharts.chart.Highchart;
import com.hyjavacharts.model.common.Color;
import com.hyjavacharts.model.common.General;
import com.hyjavacharts.model.common.LinearGradient;
import com.hyjavacharts.model.common.Stop;
import com.hyjavacharts.model.highcharts.ChartOptions;
import com.hyjavacharts.model.highcharts.constants.AxisType;
import com.hyjavacharts.model.highcharts.constants.SeriesType;
import com.hyjavacharts.model.highcharts.series.SeriesArea;
import com.hyjavacharts.model.highcharts.series.SeriesWindbarb;
import com.webonlineapp.hyjavachartsdemo12.charts.DemoChartBase;

public class WindBarb extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highchart configure() {
		Highchart highChart = new Highchart();
		ChartOptions chartOptions = highChart.getChartOptions();
		
		chartOptions.getTitle().setText("Observed wind in Vik, 10. July 2024");
		chartOptions.getSubtitle().setText("Source: <a href=\"https://seklima.met.no/\" target=\"_blank\">seklima</a>");
		
		chartOptions.getXAxisSingle().setType(AxisType.DATETIME).setOffset(40);
		chartOptions.getYAxisSingle().getTitle().setText("Wind speed (m/s)");
		
		chartOptions.getPlotOptions().getSeries()
			.setPointStart("2024-07-11")
			.setPointInterval(36e5);
		
		SeriesWindbarb seriesWindbarb = new SeriesWindbarb();
		seriesWindbarb.setType(SeriesType.WINDBARB)
			.setName("Wind")
			.setShowInLegend(false)
			.setColor(chartOptions.getColors().get(1))
			.setDataAsJsValue(getSeriesData())
			.getTooltip().setValueSuffix(" m/s");
		seriesWindbarb.setDataAsJsValue(getSeriesData());
		chartOptions.getSeries().add(seriesWindbarb);
		
		SeriesArea seriesArea = new SeriesArea();
		seriesArea.getLabel().setEnabled(false);
		seriesArea
			.setType(SeriesType.AREA)
			.setKeys(Arrays.asList("y"))
			.setDataAsJsValue(getSeriesData())
			.setName("Wind speed")
			.setColor(chartOptions.getColors().get(0))
			.setFillColor(new Color(new LinearGradient(0, 0, 0, 1), 
				Arrays.asList(new Stop(0, chartOptions.getColors().get(0)), 
								new Stop(1, new Color().setColorValue("new Highcharts.Color(Highcharts.getOptions().colors[0]).setOpacity(0.25).get()")))))
			.getTooltip().setValueSuffix(" m/s");
		seriesArea.getStates().getInactive().setOpacity(1);
		chartOptions.getSeries().add(seriesArea);
		
		return highChart;
	}
	
	/**
	 * 
	 * @param series
	 * @return
	 */
	private General getSeriesData() {
    	return new General("[\r\n"
    			+ "    [2.3, 147],\r\n"
    			+ "    [2.0, 120],\r\n"
    			+ "    [1.6, 90],\r\n"
    			+ "    [2.8, 127],\r\n"
    			+ "    [3.6, 131],\r\n"
    			+ "    [2.1, 132],\r\n"
    			+ "    [5.6, 132],\r\n"
    			+ "    [4.0, 121],\r\n"
    			+ "    [5.4, 129],\r\n"
    			+ "    [6.6, 134],\r\n"
    			+ "    [7.1, 133],\r\n"
    			+ "    [5.2, 132],\r\n"
    			+ "    [2.1, 69],\r\n"
    			+ "    [2.2, 75],\r\n"
    			+ "    [1.2, 65],\r\n"
    			+ "    [1.5, 87],\r\n"
    			+ "    [5.1, 133],\r\n"
    			+ "    [4.6, 130],\r\n"
    			+ "    [5.4, 138],\r\n"
    			+ "    [1.2, 70],\r\n"
    			+ "    [2.1, 139],\r\n"
    			+ "    [2.0, 126],\r\n"
    			+ "    [0.5, 29],\r\n"
    			+ "    [1.0, 108]\r\n"
    			+ "]");
    }
}
